from __future__ import annotations

import logging
from collections.abc import Hashable
from pathlib import Path
from typing import Tuple, Dict, Any, List, TYPE_CHECKING

import json
import yaml
from yaml.loader import SafeLoader
from checkov.common.parsers.json.decoder import SimpleDecoder
from checkov.common.resource_code_logger_filter import add_resource_code_filter_to_logger
from checkov.common.util.consts import END_LINE, START_LINE, MAX_IAC_FILE_SIZE
from checkov.common.util.file_utils import read_file_with_any_encoding

if TYPE_CHECKING:
    from yaml import MappingNode

logger = logging.getLogger(__name__)
add_resource_code_filter_to_logger(logger)


def loads(content: str) -> list[dict[str, Any]]:
    """
    Load the given JSON string
    """

    content = "[" + content + "]"
    content = content.replace('}{', '},{')
    content = content.replace('}\n{', '},\n{')

    template: list[dict[str, Any]] = yaml.load(content, Loader=SafeLineLoader)  # nosec  # custom safe loader

    # Convert an empty file to an empty list
    if template is None:
        template = []

    return template


def load(filename: Path) -> Tuple[List[Dict[str, Any]], List[Tuple[int, str]]]:
    """
    Load the given JSON file
    """

    content = read_file_with_any_encoding(file_path=filename)

    if not all(key in content for key in ("apiVersion", "kind")):
        return [{}], []

    file_lines = [(idx + 1, line) for idx, line in enumerate(content.splitlines(keepends=True))]
    file_size = len(content)

    if file_size > MAX_IAC_FILE_SIZE:
        # large JSON files take too much time, when parsed with `pyyaml`, compared to a normal 'json.loads()'
        # with start/end line numbers of 0 takes only a few seconds
        logging.info(
            f"File {filename} has a size of {file_size} which is bigger than the supported 50mb, "
            "therefore file lines will default to 0."
            "This limit can be adjusted via the environment variable 'CHECKOV_MAX_IAC_FILE_SIZE'."
        )
        return json.loads(content, cls=CustomDecoder), file_lines

    template = loads(content)

    return template, file_lines


class SafeLineLoader(SafeLoader):
    def construct_mapping(self, node: MappingNode, deep: bool = False) -> dict[Hashable, Any]:
        mapping = super().construct_mapping(node, deep=deep)
        # Add 1 so line numbering starts at 1
        # mapping['__line__'] = node.start_mark.line + 1
        mapping[START_LINE] = node.start_mark.line + 1
        mapping[END_LINE] = node.end_mark.line + 1
        return mapping


class CustomDecoder(SimpleDecoder):
    def object_hook(self, obj: dict[str, Any]) -> Any:
        obj[START_LINE] = 0
        obj[END_LINE] = 0
        return obj



from checkov.common.models.enums import CheckCategories
from checkov.arm.base_resource_value_check import BaseResourceValueCheck


class SynapseWorkspaceEnablesDataExfilProtection(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that Synapse workspace has data_exfiltration_protection_enabled"
        id = "CKV_AZURE_157"
        supported_resources = ["Microsoft.Synapse/workspaces"]
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return 'properties/dataExfiltrationProtectionEnabled'


check = SynapseWorkspaceEnablesDataExfilProtection()



from checkov.cloudformation.checks.resource.BaseCloudsplainingIAMCheck import BaseCloudsplainingIAMCheck


class cloudsplainingPrivilegeEscalation(BaseCloudsplainingIAMCheck):

    def __init__(self):
        name = "Ensure IAM policies does not allow privilege escalation"
        id = "CKV_AWS_110"
        super().__init__(name=name, id=id)

    def cloudsplaining_analysis(self, policy):
        escalations = policy.allows_privilege_escalation
        flattened_escalations: list[str] = []
        if escalations:
            for escalation in escalations:
                if isinstance(escalation, dict):
                    flattened_escalations.extend(escalation.get('actions'))
                else:
                    flattened_escalations.append(escalation)
        return flattened_escalations


check = cloudsplainingPrivilegeEscalation()



from typing import Dict, List, Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class AKSNodePublicIpDisabled(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure AKS cluster nodes do not have public IP addresses"
        id = "CKV_AZURE_143"
        supported_resources = ['azurerm_kubernetes_cluster']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, List[Any]]) -> CheckResult:
        if 'default_node_pool' in conf:
            default_node_pool = conf['default_node_pool'][0]
            if isinstance(default_node_pool, dict):
                if default_node_pool.get('enable_node_public_ip') == [True] or default_node_pool.get('node_public_ip_enabled') == [True]:
                    return CheckResult.FAILED

        return CheckResult.PASSED

    def get_evaluated_keys(self) -> List[str]:
        return ['default_node_pool/[0]/enable_node_public_ip', 'default_node_pool/[0]/node_public_ip_enabled']


check = AKSNodePublicIpDisabled()



from __future__ import annotations

from typing import Dict, List, Any
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceCheck


class ACRUseSignedImages(BaseResourceCheck):

    def __init__(self):
        # IN ARM - Set properties.policies.trustPolicy.status to enabled, set
        # properties.policies.trustPolicy.type to Notary
        # This is the default behaviour by the tf provider when the trust policy is enabled
        name = "Ensures that ACR uses signed/trusted images"
        id = "CKV_AZURE_164"
        supported_resources = ("azurerm_container_registry",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, List[Any]]) -> CheckResult:
        if 'trust_policy_enabled' in conf:
            trust_policy_enabled = conf.get('trust_policy_enabled')
            if isinstance(trust_policy_enabled, list) and trust_policy_enabled == [True]:
                return CheckResult.PASSED

        if 'trust_policy' in conf:
            trust_policy = conf['trust_policy'][0]
            if isinstance(trust_policy, dict) and trust_policy.get('enabled') == [True]:
                return CheckResult.PASSED

        return CheckResult.FAILED

    def get_evaluated_keys(self) -> List[str]:
        return ['trust_policy_enabled', 'trust_policy/enabled']


check = ACRUseSignedImages()



from __future__ import annotations

import logging
from typing import Any

import requests

from checkov.common.util.env_vars_config import env_vars_config


class ProxyClient:
    def __init__(self) -> None:
        self.identity = env_vars_config.PROXY_HEADER_VALUE
        self.proxy_ca_path = env_vars_config.PROXY_CA_PATH
        if self.proxy_ca_path is None:
            logging.warning("[ProxyClient] CA certificate path is missing")

    def get_session(self) -> requests.Session:
        if not env_vars_config.PROXY_URL:
            logging.warning('Please provide "PROXY_URL" env var')
        proxy_url = env_vars_config.PROXY_URL
        session = requests.Session()
        proxies = {
            "http": proxy_url,
            "https": proxy_url,
        }
        session.proxies.update(proxies)  # type: ignore
        return session

    def update_request_header(self, request: requests.Request) -> None:
        if env_vars_config.PROXY_HEADER_VALUE:
            request.headers[env_vars_config.PROXY_HEADER_VALUE] = self.identity

    def send_request(self, request: requests.Request) -> requests.Response:
        session = self.get_session()
        self.update_request_header(request=request)
        prepared_request = session.prepare_request(request)
        return session.send(prepared_request, verify=self.proxy_ca_path)


def call_http_request_with_proxy(request: requests.Request) -> Any:
    proxy_client = ProxyClient()
    return proxy_client.send_request(request=request)



from typing import Optional, Any, Dict, Set, Tuple, Union, List
import ipaddress

from checkov.common.graph.checks_infra.enums import Operators
from checkov.common.checks_infra.solvers.attribute_solvers.base_attribute_solver import BaseAttributeSolver


class CIDRRangeSubsetAttributeSolver(BaseAttributeSolver):
    operator = Operators.CIDR_RANGE_SUBSET  # noqa: CCE003  # a static attribute

    def _get_operation(self, vertex: Dict[str, Any], attribute: Optional[str]) -> bool:
        if attribute is None:
            return False  # Explicitly handle None attribute

        attr_val = vertex.get(attribute)
        if not attr_val:
            return False

        # Convert the solver value into sets of IPv4 and IPv6 CIDR networks
        allowed_ranges_v4, allowed_ranges_v6 = self._to_cidr_sets(self.value)
        # Convert the vertex attribute to sets of IPv4 and IPv6 CIDR networks
        vertex_ranges_v4, vertex_ranges_v6 = self._to_cidr_sets(attr_val)

        # Check if all vertex ranges are subsets of at least one of the allowed ranges
        v4_subset_check = all(
            any(vertex_cidr.subnet_of(allowed_cidr) for allowed_cidr in allowed_ranges_v4)
            for vertex_cidr in vertex_ranges_v4
        )
        v6_subset_check = all(
            any(vertex_cidr.subnet_of(allowed_cidr) for allowed_cidr in allowed_ranges_v6)
            for vertex_cidr in vertex_ranges_v6
        )

        return v4_subset_check and v6_subset_check

    @staticmethod
    def _to_cidr_sets(value: Union[str, List[str], Set[str]]) -> Tuple[Set[ipaddress.IPv4Network], Set[ipaddress.IPv6Network]]:
        """
        Converts a value (string, list, set, etc.) into separate sets of IPv4Network and IPv6Network objects.
        """
        cidr_set_v4 = set()
        cidr_set_v6 = set()
        if isinstance(value, str):
            network = ipaddress.ip_network(value, strict=False)
            if isinstance(network, ipaddress.IPv4Network):
                cidr_set_v4.add(network)
            elif isinstance(network, ipaddress.IPv6Network):
                cidr_set_v6.add(network)
        elif isinstance(value, (list, set)):
            for v in value:
                network = ipaddress.ip_network(v, strict=False)
                if isinstance(network, ipaddress.IPv4Network):
                    cidr_set_v4.add(network)
                elif isinstance(network, ipaddress.IPv6Network):
                    cidr_set_v6.add(network)
        else:
            raise ValueError(f"Unsupported type for CIDR conversion: {type(value)}")
        return cidr_set_v4, cidr_set_v6



from __future__ import annotations

from typing import Any

from checkov.ansible.checks.base_ansible_task_value_check import BaseAnsibleTaskValueCheck
from checkov.common.models.enums import CheckResult, CheckCategories


class AptForce(BaseAnsibleTaskValueCheck):
    def __init__(self) -> None:
        name = "Ensure that the force parameter is not used, as it disables signature validation and allows packages to be downgraded which can leave the system in a broken or inconsistent state"
        id = "CKV_ANSIBLE_6"
        super().__init__(
            name=name,
            id=id,
            categories=(CheckCategories.GENERAL_SECURITY,),
            supported_modules=("ansible.builtin.apt", "apt"),
            missing_block_result=CheckResult.PASSED,
        )

    def get_expected_value(self) -> Any:
        return False

    def get_inspected_key(self) -> str:
        return "force"


check = AptForce()



from __future__ import annotations

import logging
import os
from enum import Enum
from typing import Iterable, Callable, Any
from collections.abc import Collection
from pathlib import Path

from checkov.arm.parser.parser import parse
from checkov.common.runners.base_runner import filter_ignored_paths
from checkov.runner_filter import RunnerFilter

ARM_POSSIBLE_ENDINGS = [".json"]


class ArmElements(str, Enum):
    OUTPUTS = "outputs"
    PARAMETERS = "parameters"
    RESOURCES = "resources"
    VARIABLES = "variables"

    def __str__(self) -> str:
        # needed, because of a Python 3.11 change
        return self.value


def get_scannable_file_paths(root_folder: str | None = None, excluded_paths: list[str] | None = None) -> set[str]:
    """Finds ARM files"""

    file_paths: "set[str]" = set()
    if not root_folder:
        return file_paths

    for root, d_names, f_names in os.walk(root_folder):
        filter_ignored_paths(root, d_names, excluded_paths)
        filter_ignored_paths(root, f_names, excluded_paths)
        for file in f_names:
            file_ending = os.path.splitext(file)[1]
            if file_ending in ARM_POSSIBLE_ENDINGS:
                file_paths.add(os.path.join(root, file))

    return file_paths


def create_definitions(
    root_folder: str,
    files: Collection[Path] | None = None,
    runner_filter: RunnerFilter | None = None,
) -> tuple[dict[str, dict[str, Any]], dict[str, list[tuple[int, str]]]]:
    definitions: dict[str, dict[str, Any]] = {}
    definitions_raw: dict[str, list[tuple[int, str]]] = {}
    parsing_errors: list[str] = []
    runner_filter = runner_filter or RunnerFilter()

    if root_folder:
        file_paths = get_scannable_file_paths(root_folder, runner_filter.excluded_paths)
        definitions, definitions_raw, parsing_errors = get_files_definitions(files=file_paths)

    if parsing_errors:
        logging.warning(f"[arm] found errors while parsing definitions: {parsing_errors}")

    return definitions, definitions_raw


def get_files_definitions(
        files: Iterable[str],
        filepath_fn: Callable[[str], str] | None = None,
) -> tuple[dict[str, dict[str, Any]], dict[str, list[tuple[int, str]]], list[str]]:
    """Parses ARM files into its definitions and raw data"""

    definitions = {}
    definitions_raw = {}
    parsing_errors = []

    for file in files:
        result = parse(file)

        definition, definition_raw = result
        if definition is not None and definition_raw is not None:  # this has to be a 'None' check
            path = filepath_fn(file) if filepath_fn else file
            definitions[path] = definition
            definitions_raw[path] = definition_raw
        else:
            parsing_errors.append(os.path.normpath(file))

    return definitions, definitions_raw, parsing_errors


def extract_resource_name_from_resource_id_func(resource_id: str) -> str:
    '''
        Examples:
            resourceId('Microsoft.Network/virtualNetworks/', virtualNetworkName) -> virtualNetworkName
    '''
    return clean_string(resource_id.split(',')[1].split(')')[0])


def extract_resource_name_from_reference_func(reference: str) -> str:
    '''
        Examples:
                reference('storageAccountName') -> storageAccountName
                reference('myStorage').primaryEndpoints -> myStorage
                reference('myStorage', '2022-09-01', 'Full').location -> myStorage
                reference(resourceId('storageResourceGroup', 'Microsoft.Storage/storageAccounts', 'storageAccountName')), '2022-09-01') -> storageAccountName
                reference(resourceId('Microsoft.Network/publicIPAddresses', 'ipAddressName')) -> ipAddressName
    '''
    resource_name = ')'.join(reference.split('reference(', 1)[1].split(')')[:-1])
    if 'resourceId' in resource_name:
        return clean_string(
            ''.join(resource_name.split('resourceId(', 1)[1].split(')')[0]).split(',')[-1])
    else:
        return clean_string(resource_name.split(',')[0].split('/')[-1])


def clean_string(input: str) -> str:
    return input.replace("'", '').replace(" ", "")


def clean_file_path(file_path: Path) -> Path:
    path_parts = [part for part in file_path.parts if part not in (".", "..")]

    return Path(*path_parts)



from typing import Dict, Any, Optional

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.arm.base_resource_check import BaseResourceCheck


class CosmosDBAccountsRestrictedAccess(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure Cosmos DB accounts have restricted access"
        id = "CKV_AZURE_99"
        supported_resources = ('Microsoft.DocumentDB/databaseAccounts',)
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
        properties: Optional[Dict[str, Any]] = conf.get('properties')
        if properties is not None:
            if 'enableMultipleWriteLocations' not in properties or properties['enableMultipleWriteLocations']:
                self.evaluated_keys = ['enableMultipleWriteLocations']
                if 'isVirtualNetworkFilterEnabled' in properties and properties['isVirtualNetworkFilterEnabled']:
                    self.evaluated_keys.append('isVirtualNetworkFilterEnabled')
                    if 'virtualNetworkRules' in properties and properties['virtualNetworkRules']:
                        self.evaluated_keys.append('virtualNetworkRules')
                        return CheckResult.PASSED
                    if 'ipRules' in properties and properties['ipRules']:
                        self.evaluated_keys.append('ipAddressOrRange')
                        return CheckResult.PASSED
                return CheckResult.FAILED
        return CheckResult.PASSED


check = CosmosDBAccountsRestrictedAccess()



from typing import Dict, List, Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class AWSCodeGuruHasCMK(BaseResourceCheck):
    def __init__(self):
        # This is the full description of your check
        description = "Make sure that aws_codegurureviewer_repository_association has a CMK"

        # This is the Unique ID for your check
        id = "CKV_AWS_381"

        # These are the terraform objects supported by this check (ex: aws_iam_policy_document)
        supported_resources = ['aws_codegurureviewer_repository_association']

        # Valid CheckCategories are defined in checkov/common/models/enums.py
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=description, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, List[Any]]) -> CheckResult:
        if 'kms_key_details' in conf:
            kms_key_details = conf['kms_key_details'][0]
            if 'encryption_option' in kms_key_details:
                encryption_option = kms_key_details['encryption_option'][0]
                if encryption_option == 'CUSTOMER_MANAGED_CMK':
                    return CheckResult.PASSED

        return CheckResult.FAILED


check = AWSCodeGuruHasCMK()



from __future__ import annotations

from checkov.common.graph.checks_infra.enums import Operators
from checkov.common.checks_infra.solvers.resource_solvers.base_resource_solver import BaseResourceSolver


class ExistsResourcerSolver(BaseResourceSolver):
    operator = Operators.EXISTS  # noqa: CCE003  # a static attribute

    def get_operation(self, resource_type: str | None) -> bool:
        return resource_type in self.resource_types

    def _handle_result(self, result: bool, data: dict[str, str]) -> None:
        # The exists operator means that all resources that are not in the allowlist should fail,
        # and existed resources that are in the allowlist should pass, as they are the only resources allowed
        if result:
            self._passed_vertices.append(data)
        else:
            self._failed_vertices.append(data)



from __future__ import annotations

from typing import Any

from checkov.common.graph.graph_builder.consts import GraphSource
from checkov.common.graph.graph_builder.graph_components.blocks import Block


class ArmBlock(Block):
    def __init__(
        self,
        name: str,
        config: dict[str, Any],
        path: str,
        block_type: str,
        attributes: dict[str, Any],
        id: str = "",
    ) -> None:
        super().__init__(name, config, path, block_type, attributes, id, GraphSource.ARM)

    def should_run_get_inner_attributes(self, attribute_value: Any) -> bool:
        """
        this function is triggered from _extract_inner_attributes to check whether we need to run the get_inner_attributes function.
        for ARM we want to get the inner_attributes also from list[str] and only for list[dict] like the rest of the frameworks,
        specific for the 'dependsOn' attribute in a resource
        """
        return isinstance(attribute_value, dict) or (isinstance(attribute_value, list) and len(attribute_value) > 0)



from __future__ import annotations

from typing import TYPE_CHECKING

from checkov.common.images.graph.image_referencer_manager import GraphImageReferencerManager
from checkov.kubernetes.image_referencer.provider.k8s import KubernetesProvider

if TYPE_CHECKING:
    from checkov.common.images.image_referencer import Image


class KubernetesImageReferencerManager(GraphImageReferencerManager):

    def extract_images_from_resources(self) -> list[Image]:
        k8s_provider = KubernetesProvider(graph_connector=self.graph_connector)

        images = k8s_provider.extract_images_from_resources()

        return images



from __future__ import annotations

import logging
import os
import re
from collections.abc import Collection
from pathlib import Path
from typing import TYPE_CHECKING

from checkov.common.runners.base_runner import filter_ignored_paths
from checkov.runner_filter import RunnerFilter
from checkov.bicep.parser import Parser

if TYPE_CHECKING:
    from pycep.typing import BicepJson


BICEP_POSSIBLE_ENDINGS = [".bicep"]
BICEP_START_LINE = "__start_line__"
BICEP_END_LINE = "__end_line__"


def get_scannable_file_paths(
    root_folder: str | Path | None = None, files: list[str] | None = None, excluded_paths: list[str] | None = None
) -> set[Path]:
    """Finds Bicep files"""

    file_paths: set[Path] = set()

    if root_folder:
        root_path = Path(root_folder)
        file_paths = {file_path for file_path in root_path.rglob("*.bicep") if file_path.is_file()}

        if excluded_paths:
            compiled = [re.compile(p.replace(".terraform", r"\.terraform")) for p in excluded_paths]
            file_paths = {
                file_path for file_path in file_paths if not any(pattern.search(str(file_path)) for pattern in compiled)
            }
    if files:
        for file in files:
            if file.endswith(".bicep"):
                file_paths.add(Path(file))

    return file_paths


def clean_file_path(file_path: Path) -> Path:
    path_parts = [part for part in file_path.parts if part not in (".", "..")]

    return Path(*path_parts)


def get_folder_definitions(
    root_folder: str, excluded_paths: list[str] | None
) -> tuple[dict[Path, BicepJson], dict[Path, list[tuple[int, str]]], list[str]]:
    files_list: set[Path] = set()
    for root, d_names, f_names in os.walk(root_folder):
        filter_ignored_paths(root, d_names, excluded_paths)
        filter_ignored_paths(root, f_names, excluded_paths)
        for file in f_names:
            file_ending = os.path.splitext(file)[1]
            if file_ending in BICEP_POSSIBLE_ENDINGS:
                full_path = os.path.join(root, file)
                files_list.add(Path(full_path))
    parser = Parser()

    return parser.get_files_definitions(files_list)


def create_definitions(
    root_folder: str,
    files: "Collection[Path] | None" = None,
    runner_filter: RunnerFilter | None = None,
) -> tuple[dict[Path, BicepJson], dict[Path, list[tuple[int, str]]]]:
    definitions: dict[Path, BicepJson] = {}
    definitions_raw: dict[Path, list[tuple[int, str]]] = {}
    parsing_errors: list[str] = []
    runner_filter = runner_filter or RunnerFilter()

    if files:
        parser = Parser()
        definitions, definitions_raw, parsing_errors = parser.get_files_definitions(file_paths=files)

    if root_folder:
        definitions, definitions_raw, parsing_errors = get_folder_definitions(root_folder, runner_filter.excluded_paths)

    if parsing_errors:
        logging.warning(f"[bicep] found errors while parsing definitions: {parsing_errors}")

    return definitions, definitions_raw



from typing import Any, Dict, List

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class PostgreSQLFlexiServerGeoBackupEnabled(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that PostgreSQL Flexible server enables geo-redundant backups"
        id = "CKV_AZURE_136"
        supported_resources = ['azurerm_postgresql_flexible_server']
        categories = [CheckCategories.BACKUP_AND_RECOVERY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, List[Any]]) -> CheckResult:
        # Replicas can't have geo-redundant backups
        if conf.get('create_mode') and conf.get('create_mode')[0] == 'Replica':
            return CheckResult.PASSED
        return super().scan_resource_conf(conf)

    def get_inspected_key(self) -> str:
        return 'geo_redundant_backup_enabled'


check = PostgreSQLFlexiServerGeoBackupEnabled()



from checkov.common.models.enums import CheckCategories
from checkov.arm.base_resource_value_check import BaseResourceValueCheck


class AzureSparkPoolIsolatedComputeEnabled(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure isolated compute is enabled for Synapse Spark pools"
        id = "CKV_AZURE_242"
        supported_resources = ["Microsoft.Synapse/workspaces/bigDataPools"]
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return 'properties/isComputeIsolationEnabled'


check = AzureSparkPoolIsolatedComputeEnabled()



from __future__ import annotations

import os
from collections.abc import Collection
from enum import Enum
from typing import Callable, Any
from pathlib import Path

from checkov.common.parallelizer.parallel_runner import parallel_runner
from checkov.runner_filter import RunnerFilter
from checkov.serverless.parsers.parser import parse
from checkov.common.runners.base_runner import filter_ignored_paths

SLS_FILE_MASK = os.getenv(
    "CKV_SLS_FILE_MASK", "serverless.yml,serverless.yaml").split(",")


class ServerlessElements(str, Enum):
    PARAMS = "params"
    FUNCTIONS = "functions"
    PROVIDER = "provider"
    LAYERS = "layers"
    CUSTOM = "custom"
    PACKAGE = "package"
    PLUGINS = "plugins"
    SERVICE = "service"
    RESOURCES = "resources"

    def __str__(self) -> str:
        # needed, because of a Python 3.11 change
        return self.value


def create_definitions(
    root_folder: str,
    files: Collection[Path] | None = None,
    runner_filter: RunnerFilter | None = None,
) -> tuple[dict[str, dict[str, Any]], dict[str, list[tuple[int, str]]]]:
    definitions: dict[str, dict[str, Any]] = {}
    definitions_raw: dict[str, list[tuple[int, str]]] = {}
    runner_filter = runner_filter or RunnerFilter()

    if root_folder:
        file_paths = get_scannable_file_paths(root_folder, runner_filter.excluded_paths)
        definitions, definitions_raw = get_files_definitions(files=file_paths)

    return definitions, definitions_raw


def get_scannable_file_paths(root_folder: str | None = None, excluded_paths: list[str] | None = None) -> list[str]:
    files_list: list[str] = []

    if not root_folder:
        return files_list

    for root, d_names, f_names in os.walk(root_folder):
        # Don't walk in to "node_modules" directories under the root folder. If –for some reason–
        # scanning one of these is desired, it can be directly specified.
        if "node_modules" in d_names:
            d_names.remove("node_modules")

        filter_ignored_paths(root, d_names, excluded_paths)
        filter_ignored_paths(root, f_names, excluded_paths)
        for file in f_names:
            if file in SLS_FILE_MASK:
                files_list.append(os.path.join(root, file))

    return files_list


def get_files_definitions(
        files: list[str], filepath_fn: Callable[[str], str] | None = None
) -> tuple[dict[str, dict[str, Any]], dict[str, list[tuple[int, str]]]]:
    results = parallel_runner.run_function(_parallel_parse, files)
    definitions = {}
    definitions_raw = {}
    for file, result in results:
        if result:
            path = filepath_fn(file) if filepath_fn else file
            definitions[path], definitions_raw[path] = result

    return definitions, definitions_raw


def _parallel_parse(f: str) -> tuple[str, tuple[dict[str, Any], list[tuple[int, str]]] | None]:
    """Thin wrapper to return filename with parsed content"""
    return f, parse(f)



from __future__ import annotations

from typing import cast, Dict, Any

from checkov.common.util.consts import START_LINE, END_LINE
from checkov.common.util.suppression import collect_suppressions_for_report


ARM_COMMENT = "//"
DEFINITIONS_KEYS = ["parameters", "resources"]


def build_definitions_context(definitions: dict[str, dict[str, Any]], definitions_raw: dict[str, list[tuple[int, str]]]
                              ) -> Dict[str, Dict[str, Any]]:
    definitions_context: Dict[str, Dict[str, Any]] = {}
    for file_path_object, file_path_definitions in definitions.items():
        file_path = str(file_path_object)
        definitions_context[file_path] = {}
        for definition_attribute, resources in file_path_definitions.items():
            if definition_attribute not in DEFINITIONS_KEYS:
                continue
            definitions_context[file_path][definition_attribute] = {}
            if isinstance(resources, dict):
                for resource_key, resource_attributes in resources.items():
                    if isinstance(resource_attributes, dict):
                        add_resource_to_definitions_context(definitions_context, resource_key, resource_attributes,
                                                            definition_attribute, definitions_raw, file_path)
            elif isinstance(resources, list):
                for resource in resources:
                    if isinstance(resource, dict):
                        add_resource_to_definitions_context(definitions_context, '', resource,
                                                            definition_attribute, definitions_raw, file_path)
    return definitions_context


def add_resource_to_definitions_context(definitions_context: dict[str, dict[str, Any]], resource_key: str,
                                        resource_attributes: dict[str, Any], definition_attribute: str,
                                        definitions_raw: dict[str, Any], file_path: str) -> None:
    start_line = resource_attributes[START_LINE]
    end_line = resource_attributes[END_LINE]
    definition_resource = {"start_line": start_line, "end_line": end_line}

    if definition_attribute == "resources":
        resource_key = f"{resource_attributes.get('type')}.{resource_attributes.get('name')}"
        int_start_line = cast(int, definition_resource["start_line"])
        int_end_line = cast(int, definition_resource["end_line"])
        code_lines_for_suppressions_check = definitions_raw[file_path][int_start_line: int_end_line]
        definition_resource['skipped_checks'] = collect_suppressions_for_report(
            code_lines=code_lines_for_suppressions_check)
    else:
        definition_resource["type"] = resource_attributes.get('type')

    definition_resource["code_lines"] = definitions_raw[file_path][start_line - 1: end_line]
    definitions_context[file_path][definition_attribute][resource_key] = definition_resource



from __future__ import annotations

import logging
from collections.abc import Hashable
from pathlib import Path
from typing import Any, TYPE_CHECKING

import yaml
from yaml.loader import SafeLoader

from checkov.common.util.file_utils import read_file_with_any_encoding

if TYPE_CHECKING:
    from yaml import MappingNode


def loads(content: str) -> list[dict[str, Any]]:
    """
    Load the given YAML string
    """
    try:
        template = list(yaml.load_all(content, Loader=SafeLineLoader))
    except Exception as e:
        logging.debug(f'Fail to load yaml content, {e}')
        template = [None]
    # Convert an empty file to an empty dict
    if template is None:
        template = {}
    return template


def load(filename: str | Path, content: str | None = None) -> tuple[list[dict[str, Any]], list[tuple[int, str]]]:
    """
    Load the given YAML file
    """

    if not content:
        content = read_file_with_any_encoding(file_path=filename)

    file_lines = [(idx + 1, line) for idx, line in enumerate(content.splitlines(keepends=True))]

    template = loads(content)

    return template, file_lines


class SafeLineLoader(SafeLoader):
    def construct_mapping(self, node: MappingNode, deep: bool = False) -> dict[Hashable, Any]:
        mapping = super().construct_mapping(node, deep=deep)
        # Add 1 so line numbering starts at 1
        # mapping['__line__'] = node.start_mark.line + 1
        mapping['__startline__'] = node.start_mark.line + 1
        mapping['__endline__'] = node.end_mark.line + 1
        return mapping

    bool_values = {  # noqa: CCE003  # used to override the SafeLoader default behaviour
        'yes': True,
        'no': False,
        'true': True,
        'false': False,
        # GHA workflow files have a saved word for "on". Since we have policies inspecting the "on" section we need
        # to keep the string value.
        'on': 'on',  # type:ignore[dict-item]
        'off': False,
    }


class SafeLineLoaderGhaSchema(SafeLoader):
    def construct_mapping(self, node: MappingNode, deep: bool = False) -> dict[Hashable, Any]:
        return super().construct_mapping(node, deep=deep)

    bool_values = {  # noqa: CCE003  # used to override the SafeLoader default behaviour
        'on': 'on',  # type:ignore[dict-item]
        'off': 'off',  # type:ignore[dict-item]
        'yes': 'true',  # type:ignore[dict-item]
        'no': 'false',  # type:ignore[dict-item]
        'true': 'true',  # type:ignore[dict-item]
        'false': 'false'  # type:ignore[dict-item]
    }
